/*

    Island Mandel, a puzzle game taking place on the Mandelbrot-set
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef GIMANDEL_CONFIGDIALOG_HPP
#define GIMANDEL_CONFIGDIALOG_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "FractalDisplay.hpp"

#include "Geometry.hpp"

#include <gtk/gtk.h>

#include <cassert>

namespace islandmandel
{
class Fractal;
namespace gimandel
{

class GameWindow;

/* ************************************************************************** */
/* **************************** ConfigDialog ******************************** */
/* ************************************************************************** */

/**
 * The dialog-window where the user can choose a fractal to use as base map.
 */
class ConfigDialog
{

  /* ************************************************************************ */
  public:

    /**
     * Construct it with reference to a GameWindow.
     * @param g The game window to configure when run
     */
    explicit ConfigDialog(GameWindow& g);

    /**
     * Destroy, need to free the fractals owned by us
     */
    ~ConfigDialog();

    /**
     * Run it, possibly changing the game's configuration
     * @return False iff it was cancelled.
     */
    bool run();

  /* ************************************************************************ */
  private:

    /** The GameWindow to configure */
    GameWindow& game;

    /** The Gtk+ dialog widget */
    GtkWidget* dlg;

    // The radio-buttons for the fractal-choices
    GtkWidget* radioChecker;
    GtkWidget* radioMandelbrot;
    GtkWidget* radioJulia;

    // Widgets (needed with reference) for Julia-configuration
    GtkWidget* juliaBox;
    GtkWidget* juliaCX;
    GtkWidget* juliaCY;

    // The Mandelbrot-display to click-choose Julia parameters
    Fractal* juliaCMandelFract;
    FractalDisplay juliaCMandel;

    // The chosen fractal preview and chosen fractal in the view
    Fractal* previewFract;
    FractalDisplay preview;

    /**
     * Update the fractal preview.
     * @param redraw Redraw the display, too?
     */
    void updatePreview(bool redraw);

    /**
     * Build a fractal of the chosen type with given size
     * @param w The width
     * @param h The height
     * @return The new'ed Fractal
     */
    Fractal* buildChosenFractal(Coordinate::compT w, Coordinate::compT h) const;

    // Callbacks
    static void handle_radioToggle(GtkWidget* w, gpointer ptr);
    static void handle_juliaEntry(GtkWidget* w, gpointer ptr);
    static gboolean handle_juliaCClick(GtkWidget* w, GdkEventButton* evt,
      gpointer ptr);

};

} // namespace gimandel
} // namespace islandmandel

#endif // Header-guard
