/*

    Island Mandel, a puzzle game taking place on the Mandelbrot-set
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "GameWindow.hpp"

#include "Fractal.hpp"

namespace islandmandel
{
namespace gimandel
{

/* ************************************************************************** */
/* **************************** GameWindow ********************************** */
/* ************************************************************************** */

GameWindow::GameWindow()
  : fract(NULL), figDots(1), fig(NULL),
    map(), control(*this, map),
    w(gtk_window_new(GTK_WINDOW_TOPLEVEL)),
    configDlg(*this)
{
  assert(w);

  // Construct the UI
  gtk_window_set_title(GTK_WINDOW(w), "Island Mandel");
  gtk_window_set_resizable(GTK_WINDOW(w), FALSE);
  g_signal_connect(G_OBJECT(w), "destroy",
    G_CALLBACK(&handle_destroy), this);
  gtk_container_add(GTK_CONTAINER(w), map.getWidget());
}

GameWindow::~GameWindow()
{
  if(fract)
    delete fract;
  if(fig)
    delete fig;
}

void GameWindow::setFractal(Fractal* f)
{
  // Take it over
  if(fract)
    delete fract;
  fract=f;
  assert(fract);

  // Update sub-components
  fract->calculate();
  map.setFractal(*fract);
  control.setFractal(*fract);

  // Construct figure accordingly
  constructFigure(false);
  assert(fig);
}

bool GameWindow::show()
{
  if(!configDlg.run())
    return false;

  gtk_widget_show_all(w);
  return true;
}

void GameWindow::constructFigure(bool refresh)
{
  if(fig)
    delete fig;

  fig=new Figure(figDots, 8*MapDisplay::FIGURE_DOT_RADIUS,
    MapDisplay::FIGURE_DOT_RADIUS*2, fract->getNumberColours());
  assert(fig);

  map.setFigure(*fig);
  control.setFigure(*fig);

  if(refresh)
    map.redraw();
}

void GameWindow::handle_destroy(GtkWidget* w, gpointer ptr)
{
#ifndef NDEBUG
  assert(ptr);
  const GameWindow& me(*static_cast<const GameWindow*>(ptr));
  assert(w==me.w);
#endif
  gtk_main_quit();
}

} // namespace gimandel
} // namespace islandmandel
