/*

    Island Mandel, a puzzle game taking place on the Mandelbrot-set
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef GIMANDEL_MAPDISPLAY_HPP
#define GIMANDEL_MAPDISPLAY_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "FractalDisplay.hpp"

#include "Geometry.hpp"

#include <cassert>

namespace islandmandel
{
class Fractal;
class Figure;
namespace gimandel
{

/* ************************************************************************** */
/* **************************** MapDisplay ********************************** */
/* ************************************************************************** */

/**
 * The FractalDisplay additionally painting the Figure with correct
 * matching-colours, that is, the main map.
 */
class MapDisplay : public FractalDisplay
{

  /* ************************************************************************ */
  public:

    /**
     * The radius used for plotting figure-dots, should be minimum dot
     * distance during creation, too.
     */
    static const Coordinate::compT FIGURE_DOT_RADIUS;

    /**
     * Create it, figure/fractal empty for now
     */
    inline MapDisplay()
      : FractalDisplay()
#ifndef NDEBUG
      , fig(NULL)
#endif
      {}

    /**
     * Exchange/set the figure
     * @param f The new figure
     */
    inline void setFigure(const Figure& f)
    {
      fig=&f;
      assert(fig);
      // No redraw here, must be done explicitelly if needed
    }

  /* ************************************************************************ */
  protected:

    /**
     * Draw, also the Figure.
     * @see FractalDisplay::draw(GdkDrawable*, GdkGC*)
     */
    virtual void draw(GdkDrawable* screen, GdkGC* gc) const;

  /* ************************************************************************ */
  private:

    /** Reference to our Figure */
    const Figure* fig;

};

} // namespace gimandel
} // namespace islandmandel

#endif // Header-guard
