/*

    Island Mandel, a puzzle game taking place on the Mandelbrot-set
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef IMANDEL_CHECKER_HPP
#define IMANDEL_CHECKER_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Fractal.hpp"
#include "Figure.hpp"

namespace islandmandel
{

/* ************************************************************************** */
/* ****************************** Checker *********************************** */
/* ************************************************************************** */

/**
 * A simple "fractal" in shape of a checker-board with squares of length 1.0
 */
class Checker : public Fractal
{

  /* ************************************************************************ */
  public:

    /**
     * Default start range is (-1, -1) to (1, 1) so that 4 squares are visible.
     * @param pw Pixel width
     * @param ph Pixel height
     * @param n Number colours to use
     */
    inline Checker(Coordinate::compT pw, Coordinate::compT ph,
      Figure::colourT n =2)
      : Fractal(pw, ph, 0., 0., n, n), numCol(n)
    {}

    /**
     * @see Fractal::getNumberColours()
     */
    virtual Figure::colourT getNumberColours() const;

  /* ************************************************************************ */
  protected:

    /**
     * Calculate the checker-board
     * @see Fractal::calcArea(...) 
     */
    virtual void calcArea(floatCoordT tx, floatCoordT ty,
      floatCoordT sx, floatCoordT sy, Coordinate::compT w, Coordinate::compT h,
      Figure::colourT* px, Coordinate::compT rs) const;

  /* ************************************************************************ */
  private:

    /** Number of colours used */
    const Figure::colourT numCol;

};

} // namespace islandmandel

#endif // Header-guard
