/*

    Island Mandel, a puzzle game taking place on the Mandelbrot-set
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Test program for the basic geometry operations, that is, Coordinates and
 * transformations like rotations and scaling.
 */

#include "Test.hpp"

#include "Geometry.hpp"

#include <cstdlib>
#include <cmath>

using namespace islandmandel;
using namespace tests;

int main()
{
  // Define the coordinate (1, 1)
  Coordinate c(1, 1);

  // Build a combined scaling-rotation-transformation
  Transformation t(
    Transformation::forScaling(2.)*
    Transformation::forRotation(M_PI/2)*
    Transformation::forScaling(-1.));
  
  /* Applying this to the coordinate should yield:
   * (1, 1) -> (-1, -1) by the -1 scaling,
   * (-1, -1) -> (1, -1) by the rotation,
   * (1, -1) -> (2, -2) by the 2 scaling
   */

  // Check this!
  c=t(c);
  check(c.getX()==2 && c.getY()==-2,
    "Transformation done wrong, result is (%d, %d) instead of (2, -2)!",
    c.getX(), c.getY());
  
  return EXIT_SUCCESS;
}
